@@ -138,6 +138,61 @@ int runBackupOne(NDBT_Context* ctx, NDBT_Step* step){
   return NDBT_OK;
 }
 
+int
+runBackupLoop(NDBT_Context* ctx, NDBT_Step* step){
+  NdbBackup backup(GETNDB(step)->getNodeId()+1);
+  unsigned backupId = 0;
+  
+  int loops = ctx->getNumLoops();
+  while(!ctx->isTestStopped() && loops--)
+  {
+    if (backup.start(backupId) == -1)
+    {
+      sleep(1);
+      loops++;
+    }
+    else
+    {
+      sleep(3);
+    }
+  }
+
+  ctx->stopTest();
+  return NDBT_OK;
+}
+
+int
+runDDL(NDBT_Context* ctx, NDBT_Step* step){
+  Ndb* pNdb= GETNDB(step);
+  NdbDictionary::Dictionary* pDict = pNdb->getDictionary();
+  
+  const int tables = NDBT_Tables::getNumTables();
+  while(!ctx->isTestStopped())
+  {
+    const int tab_no = rand() % (tables);
+    NdbDictionary::Table tab = *NDBT_Tables::getTable(tab_no);
+    BaseString name= tab.getName();
+    name.appfmt("-%d", step->getStepNo());
+    tab.setName(name.c_str());
+    if(pDict->createTable(tab) == 0)
+    {
+      HugoTransactions hugoTrans(* pDict->getTable(name.c_str()));
+      if (hugoTrans.loadTable(pNdb, 10000) != 0){
+	return NDBT_FAILED;
+      }
+      
+      while(pDict->dropTable(tab.getName()) != 0 &&
+	    pDict->getNdbError().code != 4009)
+	g_err << pDict->getNdbError() << endl;
+      
+      sleep(1);
+
+    }
+  }
+  return NDBT_OK;
+}
+
+
 int runRestartInitial(NDBT_Context* ctx, NDBT_Step* step){
   NdbRestarter restarter;
 
@@ -174,10 +229,13 @@ int runVerifyOne(NDBT_Context* ctx, NDBT_Step* step){
   int result = NDBT_OK;
   int count = 0;
 
-  ndbout << *(const NDBT_Table*)ctx->getTab() << endl;
-
-  UtilTransactions utilTrans(*ctx->getTab());
-  HugoTransactions hugoTrans(*ctx->getTab());
+  const NdbDictionary::Table* tab = 
+    GETNDB(step)->getDictionary()->getTable(ctx->getTab()->getName());
+  if(tab == 0)
+    return NDBT_FAILED;
+  
+  UtilTransactions utilTrans(* tab);
+  HugoTransactions hugoTrans(* tab);
 
   do{
 
@@ -216,7 +274,7 @@ int runDropTable(NDBT_Context* ctx, NDBT_Step* step){
 #include "bank/Bank.hpp"
 
 int runCreateBank(NDBT_Context* ctx, NDBT_Step* step){
-  Bank bank;
+  Bank bank(ctx->m_cluster_connection);
   int overWriteExisting = true;
   if (bank.createAndLoadBank(overWriteExisting, 10) != NDBT_OK)
     return NDBT_FAILED;
@@ -224,7 +282,7 @@ int runCreateBank(NDBT_Context* ctx, NDBT_Step* step){
 }
 
 int runBankTimer(NDBT_Context* ctx, NDBT_Step* step){
-  Bank bank;
+  Bank bank(ctx->m_cluster_connection);
   int wait = 30; // Max seconds between each "day"
   int yield = 1; // Loops before bank returns 
 
@@ -235,7 +293,7 @@ int runBankTimer(NDBT_Context* ctx, NDBT_Step* step){
 }
 
 int runBankTransactions(NDBT_Context* ctx, NDBT_Step* step){
-  Bank bank;
+  Bank bank(ctx->m_cluster_connection);
   int wait = 10; // Max ms between each transaction
   int yield = 100; // Loops before bank returns 
 
@@ -246,7 +304,7 @@ int runBankTransactions(NDBT_Context* ctx, NDBT_Step* step){
 }
 
 int runBankGL(NDBT_Context* ctx, NDBT_Step* step){
-  Bank bank;
+  Bank bank(ctx->m_cluster_connection);
   int yield = 20; // Loops before bank returns 
   int result = NDBT_OK;
 
@@ -260,7 +318,7 @@ int runBankGL(NDBT_Context* ctx, NDBT_Step* step){
 }
 
 int runBankSum(NDBT_Context* ctx, NDBT_Step* step){
-  Bank bank;
+  Bank bank(ctx->m_cluster_connection);
   int wait = 2000; // Max ms between each sum of accounts
   int yield = 1; // Loops before bank returns 
   int result = NDBT_OK;
@@ -275,7 +333,7 @@ int runBankSum(NDBT_Context* ctx, NDBT_Step* step){
 }
 
 int runDropBank(NDBT_Context* ctx, NDBT_Step* step){
-  Bank bank;
+  Bank bank(ctx->m_cluster_connection);
   if (bank.dropBank() != NDBT_OK)
     return NDBT_FAILED;
   return NDBT_OK;
@@ -349,7 +407,7 @@ int runRestoreBankAndVerify(NDBT_Context* ctx, NDBT_Step* step){
     // To erase all tables from cache(s)
     // To be removed, maybe replaced by ndb.invalidate();
     {
-      Bank bank;
+      Bank bank(ctx->m_cluster_connection);
       
       if (bank.dropBank() != NDBT_OK){
 	result = NDBT_FAILED;
@@ -372,7 +430,7 @@ int runRestoreBankAndVerify(NDBT_Context* ctx, NDBT_Step* step){
     ndbout << "Backup " << backupId << " restored" << endl;
 
     // Let bank verify
-    Bank bank;
+    Bank bank(ctx->m_cluster_connection);
 
     int wait = 0;
     int yield = 1;
@@ -417,6 +475,15 @@ TESTCASE("BackupOne",
   VERIFIER(runVerifyOne);
   FINALIZER(runClearTable);
 }
+TESTCASE("BackupDDL", 
+	 "Test that backup and restore works on with DDL ongoing\n"
+	 "1. Backups and DDL (create,drop,table.index)"){
+  INITIALIZER(runLoadTable);
+  STEP(runBackupLoop);
+  STEP(runDDL);
+  STEP(runDDL);
+  FINALIZER(runClearTable);
+}
 TESTCASE("BackupBank", 
 	 "Test that backup and restore works during transaction load\n"
 	 " by backing up the bank"
